@@ -232,8 +232,9 @@ public abstract class PDField implements COSObjectable
     }
 
     /**
-     * sets the field to be required.
-     * 
+     * sets the flag whether the field is to be required to have a value at the time it is exported
+     * by a submit-form action.
+     *
      * @param required The new flag for required.
      */
     public void setRequired(boolean required)
@@ -242,8 +243,8 @@ public abstract class PDField implements COSObjectable
     }
 
     /**
-     * 
-     * @return true if the field is required
+     * @return true if the field is required to have a value at the time it is exported by a
+     * submit-form action.
      */
     public boolean isRequired()
     {
@@ -416,9 +417,10 @@ public abstract class PDField implements COSObjectable
     }
 
     /**
-     * This will get the single associated widget that is part of this field. This occurs when the Widget is embedded in
-     * the fields dictionary. Sometimes there are multiple sub widgets associated with this field, in which case you
-     * want to use getKids(). If the kids entry is specified, then the first entry in that list will be returned.
+     * This will get the single associated widget that is part of this field. This occurs when the Widget is
+     * embedded in the fields dictionary. Sometimes there are multiple sub widgets associated with this field,
+     * in which case you want to use {@link #getWidgets()}. If the kids entry is specified, then only first
+     * entry in that list will be returned.
      * 
      * @return The widget that is associated with this field.
      * @throws IOException If there is an error getting the widget object.
@@ -450,6 +452,45 @@ public abstract class PDField implements COSObjectable
         return retval;
     }
 
+    /**
+     * Returns the widget annotations associated with this field.
+     * 
+     * The widget annotations are returned for terminal fields (fields which
+     * do not have other fields as kids). In case of non terminal fields an 
+     * empty List is returned. 
+     * 
+     * @return The list of widget annotations.
+     */
+    public List<PDAnnotationWidget> getWidgets()
+    {
+        List<PDAnnotationWidget> widgets = new ArrayList<PDAnnotationWidget>();
+        
+        COSArray kids = (COSArray) getDictionary().getDictionaryObject(COSName.KIDS);
+        if (kids == null)
+        {
+            // the field itself is a widget
+            widgets.add(new PDAnnotationWidget(getDictionary()));
+        }
+        else
+        {
+            for (int i = 0; i < kids.size(); i++)
+            {
+                COSDictionary kidDictionary = (COSDictionary) kids.getObject(i);
+                
+                if (kidDictionary == null)
+                {
+                   continue;
+                }
+                
+                if ("Widget".equals(kidDictionary.getNameAsString(COSName.SUBTYPE)))
+                {
+                    widgets.add(new PDAnnotationWidget(kidDictionary));
+                }
+            }
+        }
+        return widgets;
+    }
+    
     /**
      * Get the parent field to this field, or null if none exists.
      * 
@@ -521,38 +562,35 @@ public abstract class PDField implements COSObjectable
     {
         List<COSObjectable> retval = null;
         COSArray kids = (COSArray) getDictionary().getDictionaryObject(COSName.KIDS);
+        
         if (kids != null)
         {
             List<COSObjectable> kidsList = new ArrayList<COSObjectable>();
             for (int i = 0; i < kids.size(); i++)
             {
                 COSDictionary kidDictionary = (COSDictionary) kids.getObject(i);
+                
                 if (kidDictionary == null)
                 {
-                    continue;
+                   continue;
                 }
-                COSDictionary parent = (COSDictionary) kidDictionary.getDictionaryObject(COSName.PARENT, COSName.P);
-                if (kidDictionary.getDictionaryObject(COSName.FT) != null
-                        || (parent != null && parent.getDictionaryObject(COSName.FT) != null))
+                
+                // Decide if the kid is field or a widget annotation.
+                // A field dictionary that does not have a partial field name (T entry)
+                // of its own shall not be considered a field but simply a Widget annotation.
+                if (kidDictionary.getDictionaryObject(COSName.T) != null)
                 {
                     PDField field = PDFieldFactory.createField(acroForm, kidDictionary);
+                        
                     if (field != null)
                     {
-                        kidsList.add(field);
+                       kidsList.add(field);
                     }
                 }
                 else if ("Widget".equals(kidDictionary.getNameAsString(COSName.SUBTYPE)))
                 {
                     kidsList.add(new PDAnnotationWidget(kidDictionary));
                 }
-                else
-                {
-                    PDField field = PDFieldFactory.createField(acroForm, kidDictionary);
-                    if (field != null)
-                    {
-                        kidsList.add(field);
-                    }
-                }
             }
             retval = new COSArrayList<COSObjectable>(kidsList, kids);
         }
@@ -647,4 +685,25 @@ public abstract class PDField implements COSObjectable
     {
         dictionary.setItem(COSName.AA, actions);
     }
+    
+    /**
+     * Set the field type.
+     * 
+     * @param fieldType the field type string must be one of "Btn", "Ch", "Tx", "Sig"
+     */
+    protected void setFieldType(String fieldType)
+    {
+        if (fieldType.compareTo("Btn") != 0 &&
+            fieldType.compareTo("Ch") != 0 &&
+            fieldType.compareTo("Tx") != 0 &&
+            fieldType.compareTo("Sig") != 0
+                )
+        {
+            throw new IllegalArgumentException("Unknown field type given " + fieldType);
+        }
+        else
+        {
+            getDictionary().setName(COSName.FT, fieldType);
+        }
+    }
 }
